import java.net.*;
import java.io.*;
import javax.net.ssl.*;


/** Retrieves an HTTPS Server's SSL Certificate Chain and
 *  adds them to a local KeyStore/TrustManager/Certificate file
 *  so the server can be validated and an SSL connection
 *  established.
 *
 *  If the Server's certificate is not able to be validated
 *  by the KeyStore the SSL connection will not succeed.
 */
public class TrustStoreManager  {
     public String keyStore = "d:/jssecacerts";
     public String keyStorePasswd = "";
     public String keyStoreAlias = "keystorealias";

     public TrustStoreManager(String keyStore,
                              String keyStorePasswd,
                              String keyStoreAlias) {
       this.keyStore = keyStore;
       this.keyStorePasswd = keyStorePasswd;
       this.keyStoreAlias = keyStoreAlias;
     }

     /** This will create a TrustManager that will trust
      *  ALL certificates and install it as the default
      *  SSLSocketFactory TrustManager.
      *
      *  <p>Use this function to replace the default
      *  TrustManager when you are connecting to an SSL
      *  site that the certificate is not trusted.
      */
     public void createTrustALLManager() {
          try {
               SSLContext sc = SSLContext.getInstance("SSL");
               sc.init(null,trustAllCerts,
                    new java.security.SecureRandom());
               HttpsURLConnection.setDefaultSSLSocketFactory(
                    sc.getSocketFactory());
          } catch (Exception e) {
          }
     }

     /** The trust ALL TrustManager. Used by createALLTrustManager()
      *  to replace the default SSLSocketFactory TrustManager.
      */
     private TrustManager[] trustAllCerts = new TrustManager[] {
          new X509TrustManager() {
               public java.security.cert.X509Certificate[]
                    getAcceptedIssuers() {
                    return null;
               }
               public boolean isClientTrusted(
                    java.security.cert.X509Certificate[] certs) {
                    return true;
               }
               public boolean isServerTrusted(
                    java.security.cert.X509Certificate[] certs) {
                    return true;
               }
               public void checkClientTrusted(
                    java.security.cert.X509Certificate[] certs, String val) {
               }
               public void checkServerTrusted(
                    java.security.cert.X509Certificate[] certs, String val) {
               }
          }
     };

     /** Retrieves the Servers SSL Certificate chain.
      *  @param hostname The SSL host to retrieve the certicate
      *          chain from, i.e. www.verisign.com.
      *  @param port The port number of the host that accepts
      *          SSL connections. This is usually 443 unless the host
      *          has been configured to accept SSL connections on
      *          another port.
      *  @returns javax.security.cert.X509Certificate[]
      *          The host server's certificate chain. Null if there
      *          was a problem retrieving the certificate chain.
      */
     public javax.security.cert.X509Certificate[] getServerCert(
          String hostname, int port) {
          javax.security.cert.X509Certificate[] serverCerts = null;
          try {
               // Create client socket
               SSLSocketFactory factory =
                    HttpsURLConnection.getDefaultSSLSocketFactory();
               SSLSocket socket =
                    (SSLSocket)factory.createSocket(hostname,port);

               // Without doing the handshake first we get
               // "javax.net.ssl.SSLPeerUnverifiedException:
               // peer not authenticated"
               // Connect to the server
               socket.startHandshake();

               // Retrieve the server's certificate chain
               serverCerts = socket.getSession().getPeerCertificateChain();

               // Close the socket
               socket.close();
          } catch (Exception e) {
               System.out.println("getServerCert(): Exception: "
                    + e.toString() + "\n" + e.getMessage());
          }
          return serverCerts;
     }


     /** Adds the specified certificate to a KeyStore file.
      *
      *  <p>The system default KeyStore, or TrustManager, file
      *  used is either jssecacerts or cacerts, located in
      *  the <java.home>/lib/security directory.
      *
      *  @param keystoreFile The KeyStore, or TrustManager, file
      *          that the Certificate is to be added to. This is
      *          either a relative path from the execution directory
      *          or an absolute system path.
      *  @param keystorePassword The password to use on the file.
      *  @param alias An alias that the Certificate will be stored
      *          under.
      *  @param cert The Certificate to add.
      *  @returns boolean True if the Certificate was successfully
      *           added, False otherwise.
      */
     public boolean addToKeyStore(javax.security.cert.Certificate cert) {
          try {
               // Create an empty keystore
               java.security.KeyStore keystore =
                    java.security.KeyStore.getInstance(
                         java.security.KeyStore.getDefaultType());

               // Read in existing keystore data. This is need
               // to initialize the KeyStore.
               FileInputStream in = new FileInputStream(keyStore);
               keystore.load(in, null);
               in.close();

               ////////////////////////////////////////
               // This block converts a
               // javax.security.cert.Certiticate to
               // the legacy java.security.cert.Certificate
               // format. Use this with problems where a
               // cast exception (javax.security.cert ->
               // java.security.cert) occurs. The newer
               // versions of the X509 certificates are
               // part of the 'javax' hierarchy and are
               // not compatible with the legacy Class
               // files under the 'java' hierarchy.
               // <code>
               ByteArrayInputStream bais =
                    new ByteArrayInputStream(cert.getEncoded());
               java.security.cert.CertificateFactory cf =
                    java.security.cert.CertificateFactory.getInstance(
                         "X.509");
               java.security.cert.Certificate certP = null;
               while (bais.available() > 0) {
                    certP = cf.generateCertificate(bais);
               }
               // </code>
               // End Certificate conversion block
               ////////////////////////////////////////

               // Add the certificate
               keystore.setCertificateEntry(
                    keyStoreAlias, certP);

               // Save the new keystore contents
               FileOutputStream out =
                    new FileOutputStream(keyStore);
               keystore.store(out, keyStorePasswd.toCharArray());
               out.close();
          } catch (Exception e) {
               System.out.println("addToKeyStore(): Exception: "
                    + e.toString() + "\n" + e.getMessage());
               return false;
          }
          return true;
     }


     /** Trusts the given site by downloading its certificate and adding it
      * to the keystore.
      */
     public void trustSite(String site, int port) {
       // Configure our KeyStore/TrustManager/Certificate file

       // Without this we get
       // "javax.net.ssl.SSLException: untrusted server cert chain"
       //
       // Creates a TrustManager that will allow us to
       // connect to the site so we can download the
       // Server's certificate
       createTrustALLManager();

       // Get the Server's certificate chain
       javax.security.cert.X509Certificate[] xc = getServerCert(site, port);

       // Add the server's certificate chain to our
       // certificate file
       for (int i=0; i < xc.length; i++) {
           addToKeyStore(xc[i]);
       }
     }
}
